"""
=======================
Raising bubble in water
=======================

In this example, we demonstate how to measure the surface tension
through a two-step minimization.

"""
import os.path
import matplotlib.pyplot as plt

from drop.io import load_image
from drop.improcessing import (fit_circle_tip,
                               detect_edges, guess_angle)
from drop.optimize import (young_laplace,
                           fit)


from drop.optimize.deviation import radial_RMS
from drop.improcessing import worthington
from drop.utils import print_parameters


######################################################################
# Setup
# -----
#
# In this step, we:
#
# - load and crop the image
# - set the parameters
# - detect the edges
# - guess some parameters: osculating circle and tilt angle.

# Load the image
image_path = os.path.join('..', '..',
                          'data', 'uEye_Image_000827.png')
zoom = ([100, 1312], [400, 1900])
image = load_image(image_path, region=zoom)

# Parameters
calib = 0.00124 / 400  # m / px
fluid_density = 1000  # kg/m3
# Arbitrary first guess for gamma
initial_surface_tension = 0.04  # N/m
surface_tension_range = (0.02, 0.1)  # N/m

# Edge detection
edges, RZ_edges = detect_edges(image)

# Guess parameters
center_Z, center_R, radius = fit_circle_tip(edges.shape,
                                            RZ_edges,
                                            method='ransac',
                                            debug=False)
theta = guess_angle(RZ_edges, center_Z, center_R)


######################################################################
# Minimization: Step 1
# --------------------
#
# Consider only the surface tension as a varying parameter.


guesses = {'surface_tension': initial_surface_tension,
           'angle': theta,
           'center_R': center_R,
           'center_Z': center_Z,
           'radius': radius
           }

print_parameters('Initial parameters', guesses)

ini_variables = (guesses['surface_tension'], )
fixed_parameters = (guesses['angle'],
                    guesses['center_R'],
                    guesses['center_Z'],
                    guesses['radius'],
                    fluid_density,
                    calib, RZ_edges)
bounds = (surface_tension_range,)
options = {'maxiter': 10,
           'ftol': 1e-2,
           'disp': False}
# Here we use the simple model
# Note that below the method method='SLSQP' can be used also.

res = fit(ini_variables, fixed_parameters, bounds,
          method='L-BFGS-B', options=options)

print_parameters('Step 1', res)

######################################################################
# Minimization: Step 2
# --------------------
#
# Consider all the parameters in the minimization

ini_variables2 = (res.surface_tension,
                  res.angle,
                  res.center_R,
                  res.center_Z,
                  res.radius,
                  )
bounds = ((res.surface_tension-1e-2, res.surface_tension+1e-2),
          (res.angle*0.7, res.angle*1.3),
          (res.center_R-5, res.center_R+5),
          (res.center_Z-5, res.center_Z+5),
          (res.radius-10, res.radius+10),
          )
fixed_parameters = (fluid_density, calib, RZ_edges, )
# Here, we use the full model
options = {'maxiter': 100,
           'ftol': 1e-6,
           'disp': False}
res = fit(ini_variables2, fixed_parameters, bounds,
          method='L-BFGS-B', options=options)

print_parameters('Step 2', res)

######################################################################
# Plot
# ----

optimal_variables = (res.surface_tension,
                     res.angle,
                     res.center_R,
                     res.center_Z,
                     res.radius,
                     )
RZ_model = young_laplace(*optimal_variables, fluid_density,
                         calib, RZ_edges=RZ_edges, num_points=1e4)

rRMS = radial_RMS(RZ_model, RZ_edges)

# Compute the worthington number
wo_number = worthington(edges, calib, res.surface_tension, fluid_density)

plt.figure()
ax = plt.axes()
plt.imshow(image, cmap='gray')
circle = plt.Circle((center_R, center_Z), radius=radius,
                    color='c', fill=False)
ax.add_patch(circle)
plt.plot(*RZ_edges, '*g', markersize=1)
plt.plot(*RZ_model, 'r-', markersize=2)
plt.plot(center_R, center_Z, 'bo')
ttl = f'Wo = {wo_number:.2}, Gamma = {res.surface_tension:.4} N/m, RMS = {rRMS:.3}'
plt.title(ttl)
plt.show()
